package com.ibm.vap.ccfadapter;

/*
 * Source File Name : CCFAdapter.java
 * Licensed Materials - Property of IBM
 * 5655-F37
 * (C) Copyright IBM Corp. 1983, 2001. All Rights Reserved.
 */
import com.ibm.vap.generic.*;
import com.ibm.connector.*; 
/**
 * This server adapter class is used by the VisualAge Pacbase Runtime to communicate with
 * the server using CCF connectors through supported protocols.
 * <P>
 * This class implements the ServerAdapter methods using the Common Connector Framework interface delivered by IBM. You may then use any connector
 * classes that implements this interface to communicate with the server. In the following exemple we use the
 * CICS Connector (com.ibm.connector.cics.*). 
 * The InteractionSpec that retains all interaction-relevant attributes of a CCF Connector, 
 * such as a program name or interaction mode, needs to be instantiate an setup by the user.
 * The ConnectionSpec that holds all connection-relevant attributes of a CCF Connector, such as host name, port number,
 * and timeout specifications, needs also to be instatiate an setup by the user.
 * Then you will need to instantiate ccfAdapter class and set it as the current proxy adapter.
 * <P>
 * Sample code :
 * <blockquote>
 * <pre>
 *	// Instantiate the InteractionSpec class
 *	com.ibm.connector.cics.ECIInteractionSpec is = new com.ibm.connector.cics.ECIInteractionSpec();
 *	is.setCICSELUW(false);
 *	is.setPassword("USERPWD");
 *	is.setProgramName("CLECIN");
 *	is.setTPNTransactionName(true);
 *	is.setTransactionName("VEC1");
 *	is.setUserid("USERID");
 *	is.setMode(com.ibm.connector.InteractionSpecModeProperties.MODE_SEND_RECEIVE);
 *
 *	//Instantiate the ConnectionSpec class
 *	com.ibm.connector.cics.CICSConnectionSpec cs = new com.ibm.connector.cics.CICSConnectionSpec();
 *	cs.setCICSServer("CICSSTMV");
 *	cs.setConnectionTimeout(80);
 *	cs.setUnusedTimeout(30);
 *	cs.setURL("tcp://9.100.400.170:2000");
 *
 *	try {
 *		// Instantiate an Adpater class
 *		com.ibm.vap.ccfadapter.CCFAdapter ccfAdapter = new com.ibm.vap.ccfadapter.CCFAdapter(cs,is);
 *		ccfAdapter.setCommareaLength(8192);
 *		ccfAdapter.setHostCodePage("Cp1147");
 *		// Instantiate a Vap Proxy and set the adapter class
 *		com.ibm.vap.generated.proxies.CustomerProxyLv proxy = new com.ibm.vap.generated.proxies.CustomerProxyLv();
 *		proxy.setServerAdapter(ccfAdapter);
 *		proxy.selectInstances();
 *		return proxy.rows();
 *	} catch (com.ibm.vap.generic.CommunicationError ce) {
 *		return ce.getMessage();
 *	} catch (com.ibm.vap.generic.SystemError se) {
 *		return se.getMessage();
 *	}
 * </pre>
 * </blockquote>
 * 
 * @since 2.5v11
 * @see com.ibm.vap.generic.ServerAdapter
 * @version 2.5v11
 */

public class CCFAdapter extends ServerAdapter {
	private final static String copyright = "Licensed Materials - Property of IBM\n5655-F37\n(C) Copyright IBM Corp. 1983, 2001. All Rights Reserved.\nUS Government Users Restricted Rights - Use, duplication or disclosure restricted by GSA ADP\nSchedule Contract with IBM Corp.";
	protected transient java.beans.PropertyChangeSupport propertyChange;
	private ConnectionSpec connectionSpec;
	private InteractionSpec interactionSpec;
	// Used for introspection to check the exchange mode and to gain time 
	private Class interactionSpecMode;
	private java.lang.reflect.Method interactionMethod;
	
	private CCFRequester requester;
	private boolean requesterInUse = false;
	private int commareaLength;
	private String hostCodePage;
	private Communication communication;
	
/**
 * Default constructor.
 */
public CCFAdapter() {}
/**
 * Full constructor.
 * @param cs com.ibm.connector.ConnectionSpec
 * @param is com.ibm.connector.InteractionSpec
 */
public CCFAdapter(ConnectionSpec cs,InteractionSpec is) {
	connectionSpec = cs;
	interactionSpec = is;
}
public synchronized void addPropertyChangeListener(java.beans.PropertyChangeListener listener )
{
	getPropertyChange().addPropertyChangeListener(listener);
}
/**
 * Check if the interaction verb is set to MODE_SEND_RECEIVE
 * @exception CommunicationError if the mode isn't MODE_SEND_RECEIVE
 */
private void checkInteractionVerb() throws CommunicationError {
	try {
		if (interactionSpecMode == null) {
			interactionSpecMode = interactionSpec.getClass();
		}
		if (interactionMethod == null) {
			interactionMethod = interactionSpecMode.getMethod("getMode",null);
		}
		int mode = ((Integer)interactionMethod.invoke(interactionSpec,null)).intValue();
		if (mode != InteractionSpecModeProperties.MODE_SEND_RECEIVE) {
			throw new CommunicationError("Can't proceed we another mode the MODE_SEND_RECEIVE");
		}
	} catch (Exception ex) {
		throw new CommunicationError("An interanl error occured : " +ex.getMessage());
	}
}
protected CCFRequester createRequester() {
	return new CCFRequester(this);
}
/**
 * Frees resources used for the communication by this server adapter (connections, buffers, etc).
 * This method is never called by the VisualAge Pacbase Runtime,
 * but is used by the adapter when changing the connectionSpec
 * instance and it can be used by the client program.
 */
public void disconnect() {
	if (requester != null) {
		requester.release();
		requester = null;
	}
	try {
		if (communication != null )
			communication.disconnect();
			communication = null;
	}
	catch (NotConnectedException nce) {
		// No need to throw an exception, just return and do nothing more
	}
}
protected void firePropertyChange( String propertyName, Object oldValue, Object newValue )
{
	if ( propertyChange != null ) propertyChange.firePropertyChange( propertyName, oldValue, newValue );
}
/**
 * retrieves the messageLength attribut.
 * @return the commarea length
 */
public int getCommareaLength() {
	return commareaLength;
}
/**
 * retrieves the communication attribut.
 * @return com.ibm.connector.communication
 */
protected Communication getCommunication()
{
	return communication;
}
/**
 * retrieves the connectionSpec attribut.
 * @return com.ibm.connector.ConnectionSpec
 */
public ConnectionSpec getConnectionSpec() {
	return connectionSpec;
}
/**
 * retrieves the hsot code page attribut.
 * @return the value of the host encoding page code
 */
public String getHostCodePage() {
	return hostCodePage;
}
/**
 * retrieves the interactionSpec attribut.
 * @return com.ibm.connector.interactionSpec
 */
public InteractionSpec getInteractionSpec() {
	return interactionSpec;
}
/**
 * Returns the communication parameter for the specified String name.
 * This method can be overriden by the specific server adapter.
 * It is the responsibility of the specific server adapter to return the property value
 * or throw a communication exception.
 * @param name the property name
 * @return the property value
 */
public Object getProperty( String name ) {
	if ( name == null ) return null;
	
	if ( name.equals( "commareaLength" ) )
	{
		return new Integer(commareaLength);
	}
	if ( name.equals( "connectionSpec" ) )
	{
		return connectionSpec;
	}
	else if ( name.equals( "hostCodePage" ) )
	{
		return hostCodePage;
	}
	else if ( name.equals( "interactionSpec" ) )
	{
		return interactionSpec;
	}

	return null;
}
protected java.beans.PropertyChangeSupport getPropertyChange()
{
	if (propertyChange == null) propertyChange = new java.beans.PropertyChangeSupport(this);
	return propertyChange;
}
protected synchronized void releaseRequester(CCFRequester r) {
	if (requester == r) {
		requesterInUse = false;
	}
}
public synchronized void removePropertyChangeListener( java.beans.PropertyChangeListener listener )
{
	if ( propertyChange != null ) propertyChange.removePropertyChangeListener(listener);
}
/**
 * Returns an instance of requester associated to this server adapter.
 * The returned requester is initialized using the properties set in this server adapter.
 * @return a requester which can be used to send/receive requests
 * @exception CommunicationError if a CCFRequester can't be instantiate
 */
public synchronized Requester requireRequester() throws CommunicationError {
	// Try to create a communication instance 
	try {
		if (communication == null) {
			if (connectionSpec != null) {
				// Create a communication instance
				communication = connectionSpec.createCommunication();
			} else {
				throw new CommunicationError("There are no connectionSpec defined for this adapter, can't establish a connection");
			}
			// Connect
			communication.connect();
		}
	} catch (AlreadyConnectedException ace) {
		// we will use the same connection to communicate
	} catch (NoConnectionAvailableException nce) {
		throw new CommunicationError(
			"Failed to get a connection due to : " + nce.getMessage());
	} catch (LogonException le) {
		throw new CommunicationError("Failed to logon due to :" + le.getMessage());
	} catch (CommunicationException ce) {
		throw new CommunicationError("Failed to connect due to : " + ce.getMessage());
	}

	// Check the interaction mode now, because if it's not in MODE_SEND_RECEIVE we can't do
	// anything more and an error is thrown. We just support synchronous mode
	checkInteractionVerb();

	// Instanciate a requester if none exists or if the "preferred" is already in use
	if (requester == null) {
		requester = createRequester();
	}
	if (!requesterInUse) {
	    requesterInUse = true;
	    return requester;
	}
	// The prefered is currently in use : create a new one !
	return createRequester();
}
/**
 * Sets the commareaLength attribut.
 * @param length of the commarea
 */
public void setCommareaLength(int newValue)
{
	int oldValue = commareaLength;
	commareaLength = newValue;
	firePropertyChange( "commareaLength", new Integer(oldValue), new Integer(newValue));	
}
/**
 * Sets the connectionSpec attribut.
 * @param com.ibm.connector.ConnectionSpec
 */
public synchronized void setConnectionSpec(ConnectionSpec newValue)
{
	// first disconnect the old connections before replacing it
	if (connectionSpec != null && !connectionSpec.equals(newValue)) {
		disconnect();
	}
	
	ConnectionSpec oldValue = connectionSpec;
	connectionSpec = newValue;
	firePropertyChange( "connectionSpec", oldValue, newValue);	
}
/**
 * Sets the host code page attribut.
 * @param the value of the host encoding page code
 */
public void setHostCodePage(String newValue)
{
	String oldValue = hostCodePage;
	hostCodePage = newValue;
	firePropertyChange( "hostCodePage", oldValue, newValue);	
}
/**
 * Sets the interactionFactory attribut.
 * @param com.ibm.connector.InteractionSpec
 */
public synchronized void setInteractionSpec(InteractionSpec newValue) {
	InteractionSpec oldValue = interactionSpec;
	interactionSpec = newValue;
	firePropertyChange( "interactionSpec", oldValue, newValue);
	// reinitialize the introspection attributs
	interactionMethod = null;
	interactionSpecMode = null;
}
/**
 * Sets a communication parameter of specified name.
 * This method can be overriden by a specific server adapter.
 * It is the responsibility of the specific server adapter set the property value
 * or throw a communication exception.
 * @param name		the property name
 * @param newValue	the property value
 */
public void setProperty(String name, Object newValue) {
	if ( name == null ) return;

	try
	{
		if ( name.equals( "commareaLength" ) )
		{
			if ( newValue instanceof Integer )
				setCommareaLength(((Integer)newValue).intValue() );
			else
				setCommareaLength(Integer.valueOf((String)newValue).intValue() );
		}
		else if ( name.equals( "connectionSpec" ) )
			setConnectionSpec( (ConnectionSpec)newValue );
		else if ( name.equals( "hostCodePage" ) )
			setHostCodePage( (String)newValue );
		else if ( name.equals( "interactionSpec" ) )
			setInteractionSpec((InteractionSpec)newValue );
	}
	catch ( ClassCastException e ) {}
}
}
