package com.ibm.vap.ccfadapter;

/*
 * Source File Name : CCFRequester.java
 * Licensed Materials - Property of IBM
 * 5655-F37
 * (C) Copyright IBM Corp. 1983, 2002. All Rights Reserved.
 */
import com.ibm.vap.generic.*;
import com.ibm.connector.*;
/**
 * Class used to adapt the Common Connector Framework to the Visualage Pacbase Runtime.
 * This class is a subclass of the com.ibm.vap.generic.Requester.
 * @since 2.5v11
 */

public class CCFRequester extends Requester {
	private final static String copyright = "Licensed Materials - Property of IBM\n5655-F37\n(C) Copyright IBM Corp. 1983, 2002. All Rights Reserved.\nUS Government Users Restricted Rights - Use, duplication or disclosure restricted by GSA ADP\nSchedule Contract with IBM Corp.";
	protected CCFAdapter adapter;
	protected byte outBuffer[];
protected CCFRequester(CCFAdapter ccfAdapter) {
	adapter = ccfAdapter;
}
protected byte[] createOutBuffer() {
	return new byte[getMaxMessageLength()];
}
/**
 * Gets the maximum message length.
 * Return the maximum message length it requires.
 * The VisualAge Pacbase runtime garanties that no buffer passed to the function "send" will
 * be longer than this maximum.
 * If this function returns a value equals to or lower than 0, no maximum message
 * length is considered by the VisualAge Pacbase Runtime.
 */
public int getMaxMessageLength() throws CommunicationError {
	return adapter.getCommareaLength();
}
/**
 * Waits for a response associated to the last request.
 * This function must wait until at least one character has been received from the
 * VisualAge Pacbase communication monitor at the server side.
 * The received characters must be stored in the given char array, starting at the given offset and
 * stopping when either "length" characters have been copied or no more character is available for
 * reception.
 * The function must return the total number of characters currently received.
 * When this method returns the value "nbReturn", two cases can be considered :
 * - "nbReturn <= length" : only "nbReturn" characters have been currently received and are stored in the
 *  char array. A subsequent call to "receive" will likely block.
 * - "nbReturn > length" : more than "length" characters have been received, but only the first "length"
 *  have been stored in the char array. Subsequent calls to "receive" will not block until at least all the
 *  remaining characters (nbReturn-length) have been returned.
 * Calling "receive" with a length of 0 is a good way to wait for characters without getting any of them (thus,
 * without having to allocate a char array). Then "receive" can be called a second time to get all the currently
 * received characters (passing the value returned by the first call as "length" parameter).
 * Since this method must wait until characters are received, it should never return 0.
 * The only case where the returned value can be 0 is when a receive timeout has expired.
 * @exception CommunicationError is thrown if an error condition is encountered.
 */
public synchronized int receive(char[] message, int offset, int length) throws CommunicationError {
	// Converts the input message and stores it in a byte array
	try {
		String out = new String(outBuffer, offset, length, adapter.getHostCodePage());
		out.getChars(0, length, message, offset);
		return length;
	} catch (java.io.UnsupportedEncodingException uee) {
		throw new CommunicationError("Byte to Char Conversion Error", uee);
	}
}
/**
 * Checks for a response associated to the last request, without waiting.
 * The method returns the number of characters currently received.
 * When this method returns 0, it means that a subsequent call to "receive" will likely block.
 * Else, it means that a subsequent call to receive will not block.
 */
public int receiveCheck() throws CommunicationError {
	return outBuffer == null ? 0 : outBuffer.length;
}
/**
 * Releases the requester.
 * Once released, the methods of the requester have an unpredictable result and thus should not be called.
 */
public void release() {
	outBuffer = null;
	adapter.releaseRequester(this);
}
/**
 * Sends a message to the server.
 * The given message must be passed to the Visualage Pacbase communication monitor at the server side.
 * @param message to be sent to the server
 * @param offset defines the starting charactere where to start sending the message
 * @param length of the message to be sent
 * @return the number of bytes effectively sent
 * @exception CommunicationError if the execute method fails
 */
public synchronized int send(char[] message, int offset, int length) throws CommunicationError {
	try {
		// Gets the converted byte array
		byte[] inBuffer = new String(message, offset, length).getBytes(adapter.getHostCodePage());
		//Create Output Object
		outBuffer = createOutBuffer();
		//Execute
		adapter.getCommunication().execute(adapter.getInteractionSpec(), inBuffer, outBuffer);
	} catch (CommunicationException ce) {
		throw new CommunicationError(ce.getMessage());
	} catch (LogonException le) {
		throw new CommunicationError(le.getMessage());
	} catch (IllegalArgumentException iae) {
		throw new CommunicationError(iae.getMessage());
	} catch (NotConnectedException nce) {
		throw new CommunicationError(nce.getMessage());
	} catch (InvalidTransactionException ite) {
		throw new CommunicationError(ite.getMessage());
	} catch (com.ibm.connector.internal.ResourceException re) {
		throw new CommunicationError(re.getMessage());
	} catch (java.io.UnsupportedEncodingException uee) {
		throw new CommunicationError("Char to Byte Conversion Error", uee);
	}
	return length;
}
}
