package com.ibm.vap.cciadapter;

/*
 * Source File Name : CCIAdapter.java
 * Licensed Materials - Property of IBM
 * 5655-F37
 * (C) Copyright IBM Corp. 1983, 2002. All Rights Reserved.
 */
import com.ibm.vap.generic.*;
import javax.resource.cci.*;
/**
 * This server adapter class is used by the VisualAge Pacbase Runtime to communicate with
 * the server using J2EE connectors through supported protocols.
 * This class implements the ServerAdapter methods using the Common Client Interface currently delivered as a Beta version by IBM.
 * You may then use any connector classes that implements this interface to communicate with the server.
 * In the following exemple we use the CICS Connector (com.ibm.connector2.cics.*). 
 * The InteractionSpec that retains all interaction-relevant attributes of a CCI Connector, 
 * such as a program name or interaction mode, needs to be instantiate an setup by the user.
 * The ConnectionFactory that holds all connection-relevant attributes of a CCI Connector, such as host name, port number,
 * and timeout specifications, needs also to be instantiate an setup by the user.
 * Then you will need to instantiate cciAdapter class and set it as the current proxy ServerAddapter.
 * <P>
 * Sample code :
 * <blockquote>
 * <pre>
 *	com.ibm.connector2.cics.ECIManagedConnectionFactory mcf = new com.ibm.connector2.cics.ECIManagedConnectionFactory();
 *	mcf.setPassword("PASSWORD");
 *	mcf.setServerName("CICSSTMV");
 *	mcf.setTPNName("VEC1");
 *	mcf.setUserName("USER");
 *	mcf.setConnectionURL("tcp://9.100.400.170:2000");
 *	com.ibm.connector2.cics.ECIConnectionFactory cf = (com.ibm.connector2.cics.ECIConnectionFactory)mcf.createConnectionFactory();
 *
 *	com.ibm.connector2.cics.ECIConnectionSpec cs = new com.ibm.connector2.cics.ECIConnectionSpec();
 *	com.ibm.connector2.cics.ECIInteractionSpec is = new com.ibm.connector2.cics.ECIInteractionSpec();
 *	is.setCommareaLength(8192);
 *	is.setFunctionName("CLECIN");
 * 
 *	try {
 *		com.ibm.vap.cciadapter.CCIAdapter cciAdapter = new com.ibm.vap.cciadapter.CCIAdapter(cf,cs,is);
 *		cciAdapter.setHostCodePage("Cp1147");
 *		com.ibm.vap.generated.proxies.CustomerProxyLv proxy = new com.ibm.vap.generated.proxies.CustomerProxyLv();
 *		proxy.setServerAdapter(cciAdapter);
 *		proxy.selectInstances();
 *		return proxy.rows();
 *	} catch (com.ibm.vap.generic.CommunicationError ce) {
 *		return ce.getMessage();
 *	}
 * </pre> 
 * </blockquote>
 * @see com.ibm.vap.generic.ServerAdapter
 * @since 2.5v11
 * @version 2.5v11
 */

public class CCIAdapter extends ServerAdapter {
	private final static String copyright = "Licensed Materials - Property of IBM\n5655-F37\n(C) Copyright IBM Corp. 1983, 2002. All Rights Reserved.\nUS Government Users Restricted Rights - Use, duplication or disclosure restricted by GSA ADP\nSchedule Contract with IBM Corp.";
	protected transient java.beans.PropertyChangeSupport propertyChange;
	private ConnectionFactory connectionFactory;
	private String hostCodePage;
	private ConnectionSpec connectionSpec;
	private InteractionSpec interactionSpec;
	private CCIRequester preferedRequester;
	private boolean preferedRequesterInUse = false;
	// Used for introspection to check the exchange mode and to gain time 
	private Class interactionSpecClass;
	private java.lang.reflect.Method interactionLengthMethod;
	private java.lang.reflect.Method interactionVerbMethod;

/**
 * Default constructor.
 */
public CCIAdapter() {}
/**
 * Full constructor.
 * @param connectionFactory javax.resource.cci.ConnectionFactory
 * @param connection javax.resource.cci.ConnectionSpec
 * @param interaction javax.resource.cci.InteractionSpec
 */
public CCIAdapter(ConnectionFactory connectionFactory,ConnectionSpec connection,InteractionSpec interaction) {
	this.connectionFactory = connectionFactory;
	this.connectionSpec = connection;
	this.interactionSpec = interaction;
}
public synchronized void addPropertyChangeListener(java.beans.PropertyChangeListener listener )
{
	getPropertyChange().addPropertyChangeListener(listener);
}
/**
 * The InteractionVerb must be set to SYNC_SEND_RECEIVE.
 * @exception CommunicationError if the interactionVerrb isn't set to SYNC_SEND_RECEIVE
 */
private void checkInteractionVerb() throws CommunicationError {
	try {
		if (interactionSpecClass == null) {
			interactionSpecClass = interactionSpec.getClass();
		}
		if (interactionVerbMethod == null) {
			interactionVerbMethod = interactionSpecClass.getMethod("getInteractionVerb",null);
		}
		int mode = ((Integer)interactionVerbMethod.invoke(interactionSpec,null)).intValue();
		if (mode != InteractionSpec.SYNC_SEND_RECEIVE) {
			throw new CommunicationError("Can't proceed we another mode the SYNC_SEND_RECEIVE");
		}
	} catch (Exception ex) {
		throw new CommunicationError("An interanl error occured : " +ex.getMessage());
	}
}
protected Connection createConnection() {
	Connection connection = null;
	try {
		if (connectionSpec != null) {
			connection = connectionFactory.getConnection(connectionSpec);
		} else {
			connection = connectionFactory.getConnection();
		}
	} catch (javax.resource.ResourceException rse) {
		throw new CommunicationError("The following error occured trying to get a connection : " + rse.getErrorCode());
	}
	return connection;
}
protected CCIRequester createRequester(Connection connection) {
	return new CCIRequester(this, connection);
}
/**
 * Frees resources used for the communication by this server adapter (connections, buffers, etc).
 * This method is never called by the VisualAge Pacbase Runtime, but can be used by the client
 * program.
 */
public synchronized void disconnect() {
	if (preferedRequester != null) {
		preferedRequester.release();
		try {
			preferedRequester.getConnection().close();
		} catch (javax.resource.ResourceException re) {
			throw new CommunicationError(
			"The following error occured trying to close a connection : "	+ re.getErrorCode());
		}
		preferedRequester = null;
	}
}
protected void firePropertyChange( String propertyName, Object oldValue, Object newValue )
{
	if ( propertyChange != null ) propertyChange.firePropertyChange( propertyName, oldValue, newValue );
}
/**
 * Retreives by introspection the CommareaLength set in the specific IneractionSpec class
 * @return the commarea length
 */
public int getCommareaLength(){
	try {
		if (interactionSpecClass == null) {
			interactionSpecClass = interactionSpec.getClass();
		}
		if (interactionLengthMethod == null) {
			interactionLengthMethod = interactionSpecClass.getMethod("getCommareaLength",null);
		}
		return ((Integer)interactionLengthMethod.invoke(interactionSpec,null)).intValue();
	} catch (Exception ex) {
		return 0;
	}
}
/**
 * retrieves the connectionFactory attribut.
 * @return javax.resource.cci.ConnectionFactory
 */
public ConnectionFactory getConnectionFactory() {
	return connectionFactory;
}
/**
 * retrieves the connectionSpec attribut.
 * @return javax.resource.cci.ConnectionSpec
 */
public ConnectionSpec getConnectionSpec() {
	return connectionSpec;
}
/**
 * retrieves the hsot code page attribut.
 * @return the value of the host encoding page code
 */
public String getHostCodePage() {
	return hostCodePage;
}
/**
 * retrieves the interactionSpec attribut.
 * @return javax.resource.cci.InteractionSpec
 */
public InteractionSpec getInteractionSpec() {
	return interactionSpec;
}
/**
 * Returns the communication parameter for the specified String name.
 * This method can be overriden by the specific server adapter.
 * It is the responsibility of the specific server adapter to return the property value
 * or throw a communication exception.
 * @param name the property name
 * @return the property value
 */
public Object getProperty( String name ) {
	if ( name == null ) return null;
	
	if ( name.equals( "connectionFactory" ) )
	{
		return connectionFactory;
	}
	if ( name.equals( "connectionSpec" ) )
	{
		return connectionSpec;
	}
	else if ( name.equals( "hostCodePage" ) )
	{
		return hostCodePage;
	}
	else if ( name.equals( "interactionSpec" ) )
	{
		return interactionSpec;
	}

	return null;
}
protected java.beans.PropertyChangeSupport getPropertyChange()
{
	if (propertyChange == null) propertyChange = new java.beans.PropertyChangeSupport(this);
	return propertyChange;
}
protected synchronized void releaseRequester(CCIRequester r) {
	if (preferedRequester == r) {
		preferedRequesterInUse = false;
	} else {
		// The requester is not the prefered one : close the specific connection !
		try {
			r.getConnection().close();
		} catch (javax.resource.ResourceException e) {
		}
	}
}
public synchronized void removePropertyChangeListener( java.beans.PropertyChangeListener listener )
{
	if ( propertyChange != null ) propertyChange.removePropertyChangeListener(listener);
}
/**
 * Returns an  instance of requester associated to this server adapter.
 * The returned requester is initialized using the properties set in this server adapter.
 * @return a requester which can be used to send/receive requests
 * @exception CommunicationError if a CCIRequester can't be instantiate 	
 */
public synchronized Requester requireRequester() throws CommunicationError {
	// Check the interaction mode now, because if it's not in SYNC_SEND_RECEIVE we can't do
	// anything more and an error is thrown. We just support synchronous mode
	checkInteractionVerb();

	// Instanciate a requester if none exists or if the "preferred" is already in use
	if (preferedRequester == null) {
		preferedRequester = createRequester(createConnection());
	}
	if (!preferedRequesterInUse) {
		preferedRequesterInUse = true;
		return preferedRequester;
	}
	// The prefered is currently in use : create a new one !
	return createRequester(createConnection());
}
/**
 * Sets the connectionFactory attribut.
 * @param javax.resource.cci.ConnectionFactory
 */
public synchronized void setConnectionFactory(ConnectionFactory newValue) {
	if (connectionFactory != null) {
		// Call the disconnect method that will release the connection resources
		disconnect();
	}
	ConnectionFactory oldValue = connectionFactory;
	connectionFactory = newValue;
	firePropertyChange( "connectionFactory", oldValue, newValue);
}
/**
 * Sets the connectionSpec attribut.
 * @param javax.resource.cci.ConnectionSpec
 */
public synchronized void setConnectionSpec(ConnectionSpec newValue) {
	if (connectionSpec != null) {
		// Call the disconnect method that will release the connection resources
		disconnect();
	}
	ConnectionSpec oldValue = connectionSpec;
	connectionSpec = newValue;
	firePropertyChange( "connectionSpec", oldValue, newValue);	
}
/**
 * Sets the host code page attribut.
 * @param String representation of the hsot code page
 */
public void setHostCodePage(String newValue)
{
	String oldValue = hostCodePage;
	hostCodePage = newValue;
	firePropertyChange( "hostCodePage", oldValue, newValue);	
}
/**
 * Sets the interactionFactory attribut.
 * @param javax.resource.cci.InteractionSpec
 */
public void setInteractionSpec(InteractionSpec newValue) {
	InteractionSpec oldValue = interactionSpec;
	interactionSpec = newValue;
	firePropertyChange( "interactionSpec", oldValue, newValue);
	// reinitialize the introspection attributs
	interactionVerbMethod = null;
	interactionSpecClass = null;
	interactionLengthMethod = null;
}
/**
 * Sets a communication parameter of specified name.
 * This method can be overriden by a specific server adapter.
 * It is the responsibility of the specific server adapter set the property value
 * or throw a communication exception.
 * @param name		the property name
 * @param newValue	the property value
 */
public void setProperty(String name, Object newValue) {
	if ( name == null ) return;

	try
	{
		if ( name.equals( "connectionFactory" ) )
			setConnectionFactory((ConnectionFactory)newValue);
		else if ( name.equals( "connectionSpec" ) )
			setConnectionSpec((ConnectionSpec)newValue );
		else if ( name.equals( "hostCodePage" ) )
			setHostCodePage((String)newValue );
		else if ( name.equals( "interactionSpec" ) )
			setInteractionSpec((InteractionSpec)newValue );
	}
	catch ( ClassCastException e ) {}
}
}
