package com.ibm.vap.ctgadapter;

/*
 * Source File Name : CTGAdapter.java
 * Licensed Materials - Property of IBM
 * 5655-F37
 * (C) Copyright IBM Corp. 1983, 2002. All Rights Reserved.
 */
import java.util.*;
import com.ibm.ctg.client.*;
import com.ibm.vap.generic.*;
/**
 * This server adapter class is used by the VisualAge Pacbase Runtime to communicate with
 * the server using CTG connectors through CICS Transaction Gateway.
 * <P>The JavaGateway is the object that represents the logic connection between a client program and a CICS Transaction Gateway.
 * You need one instance for each CTG you want to communicate with. Then you will need to instantiate a CTGAdapter class,
 * setup its properties and set it as the current proxy adapter.
 * <P>
 * Sample code :
 * <blockquote>
 * <pre>
 *	// Instantiate the JavaGateway class
 *	com.ibm.ctg.client.JavaGateway gateway = new com.ibm.ctg.client.JavaGateway();
 *	gateway.setURL("tcp://9.101.41.177:2006");
 *
 *	try {
 *		// Instantiate an Adpater class and setup paramters.
 *		com.ibm.vap.ctgadapter.CTGAdapter ctgAdapter = new com.ibm.vap.ctgadapter.CTGAdapter(gateway);
 *		ctgAdapter.setCallType(com.ibm.ctg.client.ECIRequest.ECI_SYNC_TPN);
 *		ctgAdapter.setCommareaLength(8192);
 *		ctgAdapter.setExtendMode(com.ibm.ctg.client.ECIRequest.ECI_NO_EXTEND);
 *		ctgAdapter.setLuwToken(0);
 *		ctgAdapter.setPassword("USERPWD");
 *		ctgAdapter.setProgram("CLECIN  ");
 *		ctgAdapter.setServer("CICSSTMV");
 *		ctgAdapter.setTransid("VEC1");
 *		ctgAdapter.setUserid("USERID");
 *		ctgAdapter.setHostCodePage("Cp1147");
 *
 *		// Instantiate a Vap Proxy and set the adapter class
 *		com.ibm.vap.generated.proxies.CustomerProxyLv proxy = new com.ibm.vap.generated.proxies.CustomerProxyLv();
 *		proxy.setServerAdapter(ctgAdapter);
 *		proxy.selectInstances();
 *
 *		// Disconnect the JavaGateway when it isn't needed anymore
 *		gateway.close();
 *		return proxy.rows();
 *	} catch (com.ibm.vap.generic.CommunicationError ce) {
 *		return ce.getMessage();
 *	} catch (com.ibm.vap.generic.SystemError se) {
 *		return se.getMessage();
 *	}
 * </pre> 
 * </blockquote>
 * @since 2.5v11
 * @see com.ibm.vap.generic.ServerAdapter
 * @version 2.5v11
 */

public class CTGAdapter extends ServerAdapter {
	private final static String copyright = "Licensed Materials - Property of IBM\n5655-F37\n(C) Copyright IBM Corp. 1983, 2002. All Rights Reserved.\nUS Government Users Restricted Rights - Use, duplication or disclosure restricted by GSA ADP\nSchedule Contract with IBM Corp.";
	private JavaGateway javaGateway;
	private CTGRequester requester;
	protected transient java.beans.PropertyChangeSupport propertyChange;
	
	private int callType;
	private int commareaLength;
	private int extendMode;
	private int luwToken;
	private int messageId;
	private String password;
	private String program;
	private String server;
	private  short timeout;
	private String transId;
	private String userId;
	private String hostCodePage;
/**
 * Default constructor.
 */
public CTGAdapter() {}
/**
 * Full constructor.
 * @param gateway com.ibm.ctg.client.JavaGateway
 */
public CTGAdapter(JavaGateway gateway) {
	javaGateway = gateway;
}
public synchronized void addPropertyChangeListener(java.beans.PropertyChangeListener listener )
{
	getPropertyChange().addPropertyChangeListener(listener);
}
/**
 * Frees resources used for the communication by this server adapter (connections, buffers, etc).
 * The JavaGateway isn't closed here because it's a multithreaded object that can be used by another
 * adapter, so we let the user process the JavaGateway disconnection.
 */
public synchronized void disconnect() {

	if (requester != null) {
		requester.release();
		requester = null;
	}
}
protected void firePropertyChange( String propertyName, Object oldValue, Object newValue )
{
	if ( propertyChange != null ) propertyChange.firePropertyChange( propertyName, oldValue, newValue );
}
/**
 * Retrieves the callType attribut.
 * @return the ECI call type 
 */
public int getCallType() {
	return callType;
}
/**
 * Retrieves the commarea length property.
 * @return the COMMAREA length to transmit to the CICS program
 */
public int getCommareaLength() {
	return commareaLength;
}
/**
 * Retrieves the Timeout property.
 * @return the ECI timeout. 
 */
public short getECITimeOut() {
	return timeout;
}
/**
 * Retrieves the extended mode property.
 * @return the ECI Extend_Mode
 */
public int getExtendMode() {
	return extendMode;
}
/**
 * Retrieves the host code page property.
 * @return the value of the host encoding page code
 */
public String getHostCodePage() {
	return hostCodePage;
}
/**
 * Retrieves the javaGateway instance.
 * @return com.ibm.ctg.clientJavaGateway instance used to communicate with the server
 */
public JavaGateway getJavaGateway() {
	return javaGateway;
}
/**
 * Retrieves the luwToken property.
 * @return the LUW Token identifier
 */
public int getLuwToken() {
	return luwToken;
}
/**
 * Retrieves the message qualifier property.
 * @return the message Id set by the user
 */
public int getMessageQualifier() {
	return messageId;
}
/**
 * Retrieves the password property.
 * @return String : the password value.
 */
public String getPassword() {
	return password;
}
/**
 * Retrieves the program property.
 * @return String : the program name
 */
public String getProgram() {
	return program;
}
/**
 * Returns the communication parameter for the specified String name.
 * @param name the property name
 * @return the property value
 */
public Object getProperty( String name ) {
	if ( name == null ) return null;
	
	if ( name.equals( "callType" ) )
	{
		return new Integer(callType);
	}
	if ( name.equals( "commareaLength" ) )
	{
		return new Integer(commareaLength);
	}
	else if ( name.equals( "ECITimeOut" ) )
	{
		return new Integer(timeout);
	}
	else if ( name.equals( "extendMode" ) )
	{
		return new Integer(extendMode);
	}
	else if ( name.equals( "hostCodePage" ) )
	{
		return hostCodePage;
	}
	else if ( name.equals( "javaGateway" ) )
	{
		return javaGateway;
	}
	else if ( name.equals( "luwToken" ) )
	{
		return new Integer(luwToken);
	}
	else if ( name.equals( "messageQualifier" ) )
	{
		return new Integer(messageId);
	}
	else if ( name.equals( "password" ) )
	{
		return password;
	}
	else if ( name.equals( "program" ) )
	{
		return program;
	}
	else if ( name.equals( "server" ) )
	{
		return server;
	}
	else if ( name.equals( "transId" ) )
	{
		return transId;
	}
	else if ( name.equals( "userId" ) )
	{
		return userId;
	}




	return null;
}
protected java.beans.PropertyChangeSupport getPropertyChange()
{
	if (propertyChange == null) propertyChange = new java.beans.PropertyChangeSupport(this);
	return propertyChange;
}
/**
 * Retrieves the server property.
 * @return String : the server name
 */
public String getServer() {
	return server;
}
/**
 * Retrieves the transid property.
 * @return String the transaction ID that must be used by the CICS program
 */
public String getTransid() {
	return transId;
}
/**
 * Retrieves the userid property.
 * @return the user Id for CICS
 */
public String getUserid() {
	return userId;
}
public synchronized void removePropertyChangeListener( java.beans.PropertyChangeListener listener )
{
	if ( propertyChange != null ) propertyChange.removePropertyChangeListener(listener);
}
/**
 * Returns an instance of requester associated to this server adapter, and open the javaGateway.
 * The returned requester is initialized using the properties set in this server adapter.
 * @return a requester which can be used to send/receive requests
 * @exception CommunicationError if the CTGRequester can't be instantiate
 */
public synchronized Requester requireRequester() throws CommunicationError {
	// Can't do anything if there are no JavaGateway instance 
	if ( javaGateway == null )	{
		throw new CommunicationError("Can't instantiate a requester while no javaGateway is set.");
	}
	synchronized (javaGateway) {
		try {
			if (!javaGateway.isOpen()) {
				// Open the JavaGateway
				javaGateway.open();
			}
		} catch (java.io.IOException ioe) {
			throw new CommunicationError("Failed to open the javaGateway due to : "+ ioe.getMessage());
		}
	}
	if (requester == null) {
		requester = new CTGRequester(this);
	}
	return requester;
}
/**
 * Sets the call type property.
 * @param ECI call type
 * @exception CommunicationError if the calltype is not a SYNC type
 */
public void setCallType(int newValue) throws CommunicationError {
	if (newValue != ECIRequest.ECI_SYNC && newValue != ECIRequest.ECI_SYNC_TPN) {
		throw new CommunicationError("Can't proceed we another call type then ECI_SYNC or ECI_SYNC_TPN");
	}

	int oldValue = callType;
	callType = newValue;
	firePropertyChange( "callType", new Integer(oldValue), new Integer(newValue));
}
/**
 * Sets the Commarea length property.
 * @param the COMMAREA length that will be send to the server
 */
public void setCommareaLength(int newValue) {
	int oldValue = commareaLength;
	commareaLength = newValue;
	firePropertyChange( "commareaLength", new Integer(oldValue), new Integer(newValue));
}
/**
 * Sets the timeout property.
 * @param the ECI Timeout value
 */
public void setECITimeout(short newValue) {
	short oldValue = timeout;
	timeout = newValue;
	firePropertyChange( "eciTimeout", new Integer(oldValue), new Integer(newValue));
}
/**
 * Sets the extend mode property.
 * @param the ECI Extend_Mode 
 */
public void setExtendMode(int newValue) {
	int oldValue = extendMode;
	extendMode = newValue;
	firePropertyChange( "extendMode", new Integer(oldValue), new Integer(newValue));
}
/**
 * Sets the host code page property.
 * @param the code page of the host server
 */
public void setHostCodePage(String newValue) {
	String oldValue = hostCodePage;
	hostCodePage = newValue;
	firePropertyChange( "hostCodePage", oldValue, newValue);
}
/**
 * Sets the javaGateway instance.
 * @param instance of JavaGateway that must be used to communicate with the server
 */
public synchronized void setJavaGateway(JavaGateway newValue) {
	// first disconnect the javaGateway if any
	disconnect();

	JavaGateway oldValue = javaGateway;
	javaGateway = newValue;
	firePropertyChange( "javaGateway", oldValue, newValue);
}
/**
 * Sets the luwToken mode property.
 * @param the LUW Token identifier
 */
public void setLuwToken(int newValue) {
	int oldValue = luwToken;
	luwToken = newValue;
	firePropertyChange( "luwToken", new Integer(oldValue), new Integer(newValue));
}
/**
 * Sets the message qualifier property.
 * @param the message Id
 */
public void setMessageQualifier(int newValue) {
	int oldValue = messageId;
	messageId = newValue;
	firePropertyChange( "messageId", new Integer(oldValue), new Integer(newValue));
}
/**
 * Sets the password property.
 * @param the password for the CICS user Id
 */
public void setPassword(String newValue) {
	String oldValue = password;
	password = newValue;
	firePropertyChange( "password", oldValue, newValue);
}
/**
 * Sets the program property.
 * @param the CICS program name
 */
public void setProgram(String newValue) {
	String oldValue = program;
	program = newValue;
	firePropertyChange( "program", oldValue, newValue);
}
/**
 * Sets a communication parameter of specified name.
 * @param name		the property name
 * @param newValue	the property value
 */
public void setProperty(String name, Object newValue) {
	if ( name == null ) return;

	try
	{
		if ( name.equals( "callType" ) )
		{
			if ( newValue instanceof Integer )
				setCallType(((Integer)newValue).intValue() );
			else
				setCallType(Integer.valueOf((String)newValue).intValue() );
		}
		else if ( name.equals( "commareaLength" ) )
		{
			if ( newValue instanceof Integer )
				setCommareaLength(((Integer)newValue).intValue() );
			else
				setCommareaLength(Integer.valueOf((String)newValue).intValue() );
		}
		else if ( name.equals( "ECITimeOut" ) )
		{
			if ( newValue instanceof Integer )
				setECITimeout(((Integer)newValue).shortValue() );
			else
				setECITimeout(Integer.valueOf((String)newValue).shortValue() );
		}
		else if ( name.equals( "extendMode" ) )
		{
			if ( newValue instanceof Integer )
				setExtendMode(((Integer)newValue).intValue() );
			else
				setExtendMode(Integer.valueOf((String)newValue).intValue() );
		}
		else if ( name.equals( "hostCodePage" ) )
			setHostCodePage((String)newValue);
		else if ( name.equals( "javaGateway" ) )
			setJavaGateway((JavaGateway)newValue );
		else if ( name.equals( "luwToken" ) )
		{
			if ( newValue instanceof Integer )
				setLuwToken(((Integer)newValue).intValue() );
			else
				setLuwToken(Integer.valueOf((String)newValue).intValue() );
		}
		else if ( name.equals( "messageQualifier" ) )
		{
			if ( newValue instanceof Integer )
				setMessageQualifier(((Integer)newValue).intValue() );
			else
				setMessageQualifier(Integer.valueOf((String)newValue).intValue() );
		}
		else if ( name.equals( "password" ) )
			setPassword((String)newValue );
		else if ( name.equals( "program" ) )
			setProgram((String)newValue );
		else if ( name.equals( "server" ) )
			setServer((String)newValue );
		else if ( name.equals( "transId" ) )
			setTransid((String)newValue );
		else if ( name.equals( "userId" ) )
			setUserid((String)newValue );
	}
	catch ( ClassCastException e ) {}
}
/**
 * Sets the server property.
 * @param the server name
 */
public void setServer(String newValue) {
	String oldValue = server;
	server = newValue;
	firePropertyChange( "server", oldValue, newValue);
}
/**
 * Sets the transid property.
 * @param the transaction ID that must be used by the CICS program
 */
public void setTransid(String newValue) {
	String oldValue = transId;
	transId = newValue;
	firePropertyChange( "transId", oldValue, newValue);
}
/**
 * Sets the user property.
 * @param the user Id for CICS
 */
public void setUserid(String newValue) {
	String oldValue = userId;
	userId = newValue;
	firePropertyChange( "userId", oldValue, newValue);
}
}
