package com.ibm.vap.ctgadapter;

/*
 * Source File Name : CTGAdapter.java
 * Licensed Materials - Property of IBM
 * 5655-F37
 * (C) Copyright IBM Corp. 1983, 2002. All Rights Reserved.
 */
import com.ibm.vap.generic.*;
import com.ibm.ctg.client.*;

/**
 * Class used to adapt the CTG connectors to the Visualage Pacbase Runtime.
 * This class is a subclass of the com.ibm.vap.generic.Requester.
 * @since 2.5v11
 */
public class CTGRequester extends Requester {
	private final static String copyright = "Licensed Materials - Property of IBM\n5655-F37\n(C) Copyright IBM Corp. 1983, 2002. All Rights Reserved.\nUS Government Users Restricted Rights - Use, duplication or disclosure restricted by GSA ADP\nSchedule Contract with IBM Corp.";
	private CTGAdapter adapter;
	private ECIRequest eciRequest;
/**
 * Default constructor.
 */
protected CTGRequester() {}
/**
 * Full constructor.
 */
protected CTGRequester(CTGAdapter ctgAdapter) {
	adapter = ctgAdapter;
}
/**
 * Gets the maximum message length.
 * Return the maximum message length it requires.
 * The VisualAge Pacbase runtime garanties that no buffer passed to the function "send" will
 * be longer than this maximum.
 * If this function returns a value equals to or lower than 0, no maximum message
 * length is considered by the VisualAge Pacbase Runtime.
 */
public int getMaxMessageLength() throws CommunicationError { return adapter.getCommareaLength(); }
/**
 * Instantiate a new ECIRequest according to the properties set
 * in the CTGAdapter class.
 **/
protected void initECIRequest(){
	// The ECIRequest hasn't been released, so we do not need to instantiate it
	if (eciRequest != null)
		return;
	eciRequest = new ECIRequest(adapter.getCallType(),adapter.getServer(),adapter.getUserid(),adapter.getPassword(),
			adapter.getProgram(),adapter.getTransid(),null,0,adapter.getExtendMode(),adapter.getLuwToken());
	eciRequest.setMessageId(adapter.getMessageQualifier());
	eciRequest.setECITimeout(adapter.getECITimeOut());
	eciRequest.Commarea_Length = adapter.getCommareaLength();
}
/**
 * Waits for a response associated to the last request.
 * This function must wait until at least one character has been received from the
 * VisualAge Pacbase communication monitor at the server side.
 * The received characters must be stored in the given char array, starting at the given offset and
 * stopping when either "length" characters have been copied or no more character is available for
 * reception.
 * The function must return the total number of characters currently received.
 * When this method returns the value "nbReturn", two cases can be considered :
 * - "nbReturn <= length" : only "nbReturn" characters have been currently received and are stored in the
 *  char array. A subsequent call to "receive" will likely block.
 * - "nbReturn > length" : more than "length" characters have been received, but only the first "length"
 *  have been stored in the char array. Subsequent calls to "receive" will not block until at least all the
 *  remaining characters (nbReturn-length) have been returned.
 * Calling "receive" with a length of 0 is a good way to wait for characters without getting any of them (thus,
 * without having to allocate a char array). Then "receive" can be called a second time to get all the currently
 * received characters (passing the value returned by the first call as "length" parameter).
 * Since this method must wait until characters are received, it should never return 0.
 * The only case where the returned value can be 0 is when a receive timeout has expired.
 * @exception CommunicationError is thrown if an error condition is encountered.
 */
public synchronized int receive(char[] message, int offset, int length) throws CommunicationError {
	if (eciRequest.Commarea != null && eciRequest.Cics_Rc == 0) {
		try {
			String out = new String(eciRequest.Commarea, offset, length, adapter.getHostCodePage());
			out.getChars(0, length, message, offset);
			return length;
		} catch (java.io.UnsupportedEncodingException uee) {
			throw new CommunicationError("Byte to Char Conversion Error", uee);
		}
	} else {
		throw new CommunicationError(eciRequest.Cics_Rc + " : " + eciRequest.getCicsRcString());
	}
}
/**
 * Checks for a response associated to the last request, without waiting.
 * The method returns the number of characters currently received.
 * When this method returns 0, it means that a subsequent call to "receive" will likely block.
 * Else, it means that a subsequent call to receive will not block.
 */
public int receiveCheck() {
	if (eciRequest.Commarea != null && eciRequest.Cics_Rc == 0) {
		return eciRequest.Commarea == null ? 0 : eciRequest.Commarea.length;
	} else {
		return 0;
	}
}
/**
 * This method will free the ECIRequest instance.
 **/
public void release() {
	eciRequest = null;
}
/**
 * Sends a message to the server.
 * The given message must be passed to the Visualage Pacbase communication monitor at the server side.
 * @param message to be sent to the server
 * @param offset defines the starting charactere where to start sending the message
 * @param length of the message to be sent
 * @return the number of bytes effectively sent
 * @exception CommunicationError if the flow method fails
 */
public synchronized int send(char[] message, int offset, int length) throws CommunicationError {
	// First try to create an ECIRequest instance
	initECIRequest();

	// Puts the input message in an byte array to send to the server
	try {
		eciRequest.Commarea = new String(message, offset, length).getBytes(adapter.getHostCodePage());
	} catch (java.io.UnsupportedEncodingException uee) {
		throw new CommunicationError("Char to Byte Conversion Error", uee);
	}

	//Sets the length of the message to send
	eciRequest.setCommareaOutboundLength(length);

	//Execute
	try {
		adapter.getJavaGateway().flow(eciRequest);
	} catch (java.io.IOException ioe) {
		throw new CommunicationError("Failed to execute to communication with the CTG Connectors due to : " + ioe.getMessage());
	}
	return length;
}
}
